@@ -177,6 +177,171 @@ describe("follow-redirects", function () {
       });
   });
 
+  it("http.get to IPv4 address", function () {
+    app.get("/a", redirectsTo("/b"));
+    app.get("/b", redirectsTo("/c"));
+    app.get("/c", redirectsTo("/d"));
+    app.get("/d", redirectsTo("/e"));
+    app.get("/e", redirectsTo("/f"));
+    app.get("/f", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://127.0.0.1:3600/a", concatJson(resolve, reject)).on("error", reject);
+      }))
+      .then(function (res) {
+        assert.deepEqual(res.parsedJson, { a: "b" });
+        assert.deepEqual(res.responseUrl, "http://127.0.0.1:3600/f");
+      });
+  });
+
+  it("http.get to IPv6 address", function () {
+    app.get("/a", redirectsTo("/b"));
+    app.get("/b", redirectsTo("/c"));
+    app.get("/c", redirectsTo("/d"));
+    app.get("/d", redirectsTo("/e"));
+    app.get("/e", redirectsTo("/f"));
+    app.get("/f", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://[::1]:3600/a", concatJson(resolve, reject)).on("error", reject);
+      }))
+      .then(function (res) {
+        assert.deepEqual(res.parsedJson, { a: "b" });
+        assert.deepEqual(res.responseUrl, "http://[::1]:3600/f");
+      });
+  });
+
+  it("http.get to bracketed IPv4 address", function () {
+    var error = null;
+    try {
+      http.get("http://[127.0.0.1]:3600/a");
+    }
+    catch (err) {
+      error = err;
+    }
+    assert(error instanceof Error);
+    assert(error instanceof TypeError);
+    assert.equal(error.code, "ERR_INVALID_URL");
+    assert.equal(error.input, "http://[127.0.0.1]:3600/a");
+  });
+
+  it("http.get to bracketed IPv4 address specified as host", function () {
+    var error = null;
+    try {
+      http.get({
+        host: "[127.0.0.1]:3600",
+        path: "/a",
+      });
+    }
+    catch (err) {
+      error = err;
+    }
+    assert(error instanceof Error);
+    assert(error instanceof TypeError);
+    assert.equal(error.code, "ERR_INVALID_URL");
+  });
+
+  it("http.get to bracketed IPv4 address specified as hostname", function () {
+    var error = null;
+    try {
+      http.get({
+        hostname: "[127.0.0.1]",
+        port: 3600,
+        path: "/a",
+      });
+    }
+    catch (err) {
+      error = err;
+    }
+    assert(error instanceof Error);
+    assert(error instanceof TypeError);
+    assert.equal(error.code, "ERR_INVALID_URL");
+  });
+
+  it("http.get to bracketed hostname", function () {
+    var error = null;
+    try {
+      http.get("http://[localhost]:3600/a");
+    }
+    catch (err) {
+      error = err;
+    }
+    assert(error instanceof Error);
+    assert(error instanceof TypeError);
+    assert.equal(error.code, "ERR_INVALID_URL");
+    assert.equal(error.input, "http://[localhost]:3600/a");
+  });
+
+  it("http.get redirecting to IPv4 address", function () {
+    app.get("/a", redirectsTo("http://127.0.0.1:3600/b"));
+    app.get("/b", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://localhost:3600/a", concatJson(resolve, reject)).on("error", reject);
+      }))
+      .then(function (res) {
+        assert.deepEqual(res.parsedJson, { a: "b" });
+        assert.deepEqual(res.responseUrl, "http://127.0.0.1:3600/b");
+      });
+  });
+
+  it("http.get redirecting to IPv6 address", function () {
+    app.get("/a", redirectsTo("http://[::1]:3600/b"));
+    app.get("/b", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://localhost:3600/a", concatJson(resolve, reject)).on("error", reject);
+      }))
+      .then(function (res) {
+        assert.deepEqual(res.parsedJson, { a: "b" });
+        assert.deepEqual(res.responseUrl, "http://[::1]:3600/b");
+      });
+  });
+
+  it("http.get redirecting to bracketed IPv4 address", function () {
+    app.get("/a", redirectsTo("http://[127.0.0.1]:3600/b"));
+    app.get("/b", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://localhost:3600/a", concatJson(reject)).on("error", resolve);
+      }))
+      .then(function (error) {
+        assert(error instanceof Error);
+        assert.equal(error.code, "ERR_FR_REDIRECTION_FAILURE");
+
+        var cause = error.cause;
+        assert(cause instanceof Error);
+        assert(cause instanceof TypeError);
+        assert.equal(cause.code, "ERR_INVALID_URL");
+        assert.equal(cause.input, "http://[127.0.0.1]:3600/b");
+      });
+  });
+
+  it("http.get redirecting to bracketed hostname", function () {
+    app.get("/a", redirectsTo("http://[localhost]:3600/b"));
+    app.get("/b", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://localhost:3600/a", concatJson(reject)).on("error", resolve);
+      }))
+      .then(function (error) {
+        assert(error instanceof Error);
+        assert.equal(error.code, "ERR_FR_REDIRECTION_FAILURE");
+
+        var cause = error.cause;
+        assert(cause instanceof Error);
+        assert(cause instanceof TypeError);
+        assert.equal(cause.code, "ERR_INVALID_URL");
+        assert.equal(cause.input, "http://[localhost]:3600/b");
+      });
+  });
+
   it("http.get with response event", function () {
     app.get("/a", redirectsTo("/b"));
     app.get("/b", redirectsTo("/c"));
@@ -202,8 +367,8 @@ describe("follow-redirects", function () {
     try {
       http.get("/relative");
     }
-    catch (e) {
-      error = e;
+    catch (err) {
+      error = err;
     }
     assert(error instanceof Error);
     assert(error instanceof TypeError);
@@ -211,6 +376,24 @@ describe("follow-redirects", function () {
     assert.equal(error.input, "/relative");
   });
 
+  it("redirect to URL with fragment", function () {
+    app.get("/a", redirectsTo("/b#abc"));
+    app.get("/b", redirectsTo("/c#def"));
+    app.get("/c", redirectsTo("/d#ghi"));
+    app.get("/d", redirectsTo("/e#jkl"));
+    app.get("/e", redirectsTo("/f#mno"));
+    app.get("/f", sendsJson({ a: "b" }));
+
+    return server.start(app)
+      .then(asPromise(function (resolve, reject) {
+        http.get("http://localhost:3600/a", concatJson(resolve, reject)).on("error", reject);
+      }))
+      .then(function (res) {
+        assert.deepEqual(res.parsedJson, { a: "b" });
+        assert.deepEqual(res.responseUrl, "http://localhost:3600/f#mno");
+      });
+  });
+
   it("should return with the original status code if the response does not contain a location header", function () {
     app.get("/a", function (req, res) {
       res.status(307).end();
@@ -269,7 +452,7 @@ describe("follow-redirects", function () {
         req._currentRequest.emit("connect", "r", "s", "h");
       }))
       .then(function (args) {
-        req.abort();
+        req.destroy();
         assert.equal(args.response, "r");
         assert.equal(args.socket, "s");
         assert.equal(args.head, "h");
@@ -300,7 +483,7 @@ describe("follow-redirects", function () {
         switch (error.cause.code) {
         // Node 17+
         case "ERR_INVALID_URL":
-          assert.equal(error.message, "Redirected request failed: Invalid URL");
+          assert(/^Redirected request failed: Invalid URL/.test(error.message));
           break;
         // Older Node versions
         case "ERR_UNESCAPED_CHARACTERS":
@@ -312,23 +495,6 @@ describe("follow-redirects", function () {
       });
   });
 
-  it("emits an error when url.resolve fails", function () {
-    app.get("/a", redirectsTo("/b"));
-    var urlResolve = url.resolve;
-    url.resolve = function () {
-      throw new Error();
-    };
-
-    return server.start(app)
-      .then(asPromise(function (resolve) {
-        http.get("http://localhost:3600/a").on("error", resolve);
-      }))
-      .then(function (error) {
-        url.resolve = urlResolve;
-        assert.equal(error.code, "ERR_FR_REDIRECTION_FAILURE");
-      });
-  });
-
   it("emits an error when the request fails for another reason", function () {
     app.get("/a", function (req, res) {
       res.socket.write("HTTP/1.1 301 Moved Permanently\r\n");
@@ -429,7 +595,7 @@ describe("follow-redirects", function () {
           req.on("socket", function () {
             assert(req.socket instanceof net.Socket);
             req.setTimeout(3000, function () {
-              req.abort();
+              req.destroy();
               resolve();
             });
           });
@@ -466,7 +632,7 @@ describe("follow-redirects", function () {
           });
           req.on("error", reject);
           req.setTimeout(1000, function () {
-            req.abort();
+            req.destroy();
             resolve();
           });
         }));
@@ -485,7 +651,7 @@ describe("follow-redirects", function () {
           });
           req.on("error", reject);
           req.setTimeout(2000, function () {
-            req.abort();
+            req.destroy();
             resolve();
           });
         }));
@@ -505,7 +671,7 @@ describe("follow-redirects", function () {
           var callbacks = 0;
           function timeoutCallback() {
             if (++callbacks === 3) {
-              req.abort();
+              req.destroy();
               resolve(callbacks);
             }
           }
@@ -633,7 +799,7 @@ describe("follow-redirects", function () {
     var req = http.request("http://localhost:3600/a");
     req.setHeader("my-header", "my value");
     assert.equal(req.getHeader("my-header"), "my value");
-    req.abort();
+    req.destroy();
   });
 
   it("should provide removeHeader", function () {
@@ -916,12 +1082,16 @@ describe("follow-redirects", function () {
         .then(asPromise(function (resolve, reject) {
           http.get("http://localhost:3600/a")
             .on("response", function () { return reject(new Error("unexpected response")); })
-            .on("error", reject);
+            .on("error", resolve);
         }))
-        .catch(function (error) {
+        .then(function (error) {
           assert(error instanceof Error);
-          assert(error instanceof TypeError);
-          assert.equal(error.message, "Unsupported protocol about:");
+          assert.equal(error.message, "Redirected request failed: Unsupported protocol about:");
+
+          var cause = error.cause;
+          assert(cause instanceof Error);
+          assert(cause instanceof TypeError);
+          assert.equal(cause.message, "Unsupported protocol about:");
         });
     });
   });
@@ -976,7 +1146,7 @@ describe("follow-redirects", function () {
           return;
         }
         finally {
-          req.abort();
+          req.destroy();
         }
         throw new Error("no error");
       });
@@ -1215,10 +1385,10 @@ describe("follow-redirects", function () {
       try {
         req.write(12345678);
       }
-      catch (e) {
-        error = e;
+      catch (err) {
+        error = err;
       }
-      req.abort();
+      req.destroy();
       assert(error instanceof Error);
       assert(error instanceof TypeError);
       assert.equal(error.message, "data should be a string, Buffer or Uint8Array");
@@ -1359,6 +1529,7 @@ describe("follow-redirects", function () {
 
   [
     "Authorization",
+    "Proxy-Authorization",
     "Cookie",
   ].forEach(function (header) {
     describe("when the client passes an header named " + header, function () {
@@ -2081,15 +2252,14 @@ describe("follow-redirects", function () {
               throw new Error("no redirects!");
             },
           };
-          http.get(options, concatJson(resolve, reject)).on("error", reject);
+          http.get(options, concatJson(reject)).on("error", resolve);
         }))
-        .then(function () {
-          assert.fail("request chain should have been aborted");
-        })
-        .catch(function (error) {
+        .then(function (error) {
           assert(!redirected);
           assert(error instanceof Error);
-          assert.equal(error.message, "no redirects!");
+          assert.equal(error.message, "Redirected request failed: no redirects!");
+          assert(error.cause instanceof Error);
+          assert.equal(error.cause.message, "no redirects!");
         });
     });
 
